"""
    test_domonic
    ~~~~~~~~~~~~
    unit tests for css
"""

import unittest
import json
# import requests
# from mock import patch

from domonic.html import *
from domonic.JSON import *


# class TestCase(unittest.TestCase):

# if __name__ == '__main__':
#     unittest.main()
